from atproto_core.exceptions import AtProtocolError


class InvalidCompressedPubkeyError(AtProtocolError):
    ...


class DidKeyError(AtProtocolError):
    ...


class IncorrectMultikeyPrefixError(DidKeyError):
    ...


class IncorrectDidKeyPrefixError(DidKeyError):
    ...


class UnsupportedKeyTypeError(DidKeyError):
    ...


class UnsupportedSignatureAlgorithmError(AtProtocolError):
    ...



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.temp.checkSignupQueue`."""

    activated: bool  #: Activated.
    estimated_time_ms: t.Optional[int] = Field(default=None, alias='estimatedTimeMs')  #: Estimated time ms.
    place_in_queue: t.Optional[int] = Field(default=None, alias='placeInQueue')  #: Place in queue.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Main(base.ModelBase):
    """Definition model for :obj:`app.bsky.richtext.facet`."""

    features: t.List[
        te.Annotated[
            t.Union[
                'models.AppBskyRichtextFacet.Mention',
                'models.AppBskyRichtextFacet.Link',
                'models.AppBskyRichtextFacet.Tag',
            ],
            Field(discriminator='py_type'),
        ]
    ]  #: Features.
    index: 'models.AppBskyRichtextFacet.ByteSlice'  #: Index.

    py_type: te.Literal['app.bsky.richtext.facet'] = Field(
        default='app.bsky.richtext.facet', alias='$type', frozen=True
    )


class Mention(base.ModelBase):
    """Definition model for :obj:`app.bsky.richtext.facet`. A facet feature for actor mentions."""

    did: str  #: Did.

    py_type: te.Literal['app.bsky.richtext.facet#mention'] = Field(
        default='app.bsky.richtext.facet#mention', alias='$type', frozen=True
    )


class Link(base.ModelBase):
    """Definition model for :obj:`app.bsky.richtext.facet`. A facet feature for links."""

    uri: str  #: Uri.

    py_type: te.Literal['app.bsky.richtext.facet#link'] = Field(
        default='app.bsky.richtext.facet#link', alias='$type', frozen=True
    )


class Tag(base.ModelBase):
    """Definition model for :obj:`app.bsky.richtext.facet`. A hashtag."""

    tag: str = Field(max_length=640)  #: Tag.

    py_type: te.Literal['app.bsky.richtext.facet#tag'] = Field(
        default='app.bsky.richtext.facet#tag', alias='$type', frozen=True
    )


class ByteSlice(base.ModelBase):
    """Definition model for :obj:`app.bsky.richtext.facet`. A text segment. Start is inclusive, end is exclusive. Indices are for utf8-encoded strings."""

    byte_end: int = Field(alias='byteEnd', ge=0)  #: Byte end.
    byte_start: int = Field(alias='byteStart', ge=0)  #: Byte start.

    py_type: te.Literal['app.bsky.richtext.facet#byteSlice'] = Field(
        default='app.bsky.richtext.facet#byteSlice', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.admin.queryModerationEvents`."""

    added_labels: t.Optional[t.List[str]] = Field(
        default=None, alias='addedLabels'
    )  #: If specified, only events where all of these labels were added are returned.
    comment: t.Optional[str] = None  #: If specified, only events with comments containing the keyword are returned.
    created_after: t.Optional[str] = Field(
        default=None, alias='createdAfter'
    )  #: Retrieve events created after a given timestamp.
    created_before: t.Optional[str] = Field(
        default=None, alias='createdBefore'
    )  #: Retrieve events created before a given timestamp.
    created_by: t.Optional[str] = Field(default=None, alias='createdBy')  #: Created by.
    cursor: t.Optional[str] = None  #: Cursor.
    has_comment: t.Optional[bool] = Field(
        default=None, alias='hasComment'
    )  #: If true, only events with comments are returned.
    include_all_user_records: t.Optional[bool] = Field(
        default=False, alias='includeAllUserRecords'
    )  #: If true, events on all record types (posts, lists, profile etc.) owned by the did are returned.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.
    removed_labels: t.Optional[t.List[str]] = Field(
        default=None, alias='removedLabels'
    )  #: If specified, only events where all of these labels were removed are returned.
    report_types: t.Optional[t.List[str]] = Field(default=None, alias='reportTypes')  #: Report types.
    sort_direction: t.Optional[str] = Field(
        default='desc', alias='sortDirection'
    )  #: Sort direction for the events. Defaults to descending order of created at timestamp.
    subject: t.Optional[str] = None  #: Subject.
    types: t.Optional[
        t.List[str]
    ] = None  #: The types of events (fully qualified string in the format of com.atproto.admin#modEvent<name>) to filter by. If not specified, all events are returned.


class ParamsDict(te.TypedDict):
    added_labels: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: If specified, only events where all of these labels were added are returned.
    comment: te.NotRequired[
        t.Optional[str]
    ]  #: If specified, only events with comments containing the keyword are returned.
    created_after: te.NotRequired[t.Optional[str]]  #: Retrieve events created after a given timestamp.
    created_before: te.NotRequired[t.Optional[str]]  #: Retrieve events created before a given timestamp.
    created_by: te.NotRequired[t.Optional[str]]  #: Created by.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    has_comment: te.NotRequired[t.Optional[bool]]  #: If true, only events with comments are returned.
    include_all_user_records: te.NotRequired[
        t.Optional[bool]
    ]  #: If true, events on all record types (posts, lists, profile etc.) owned by the did are returned.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    removed_labels: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: If specified, only events where all of these labels were removed are returned.
    report_types: te.NotRequired[t.Optional[t.List[str]]]  #: Report types.
    sort_direction: te.NotRequired[
        t.Optional[str]
    ]  #: Sort direction for the events. Defaults to descending order of created at timestamp.
    subject: te.NotRequired[t.Optional[str]]  #: Subject.
    types: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: The types of events (fully qualified string in the format of com.atproto.admin#modEvent<name>) to filter by. If not specified, all events are returned.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.admin.queryModerationEvents`."""

    events: t.List['models.ComAtprotoAdminDefs.ModEventView']  #: Events.
    cursor: t.Optional[str] = None  #: Cursor.



import typing as t

import httpx

from atproto_identity.did.resolvers.base_resolver import AsyncBaseResolver, BaseResolver
from atproto_identity.exceptions import DidWebResolverError, PoorlyFormattedDidError, UnsupportedDidWebPathError

if t.TYPE_CHECKING:
    from atproto_identity.cache.base_cache import AsyncDidBaseCache, DidBaseCache

_DID_DOC_PATH = '/.well-known/did.json'


class _DidWebResolverBase:
    @staticmethod
    def _parse_web_did(did: str) -> str:
        parsed_id = ':'.join(did.split(':')[2:])
        parts = parsed_id.split(':')

        if not parts:
            raise PoorlyFormattedDidError(f'Invalid DID {did}')

        if len(parts) > 1:
            raise UnsupportedDidWebPathError(f'Unsupported DID {did}')

        path = parts[0] + _DID_DOC_PATH
        return f'https://{path}'


class DidWebResolver(_DidWebResolverBase, BaseResolver):
    def __init__(
        self,
        timeout: t.Optional[float] = None,
        cache: t.Optional['DidBaseCache'] = None,
    ) -> None:
        super().__init__(cache)

        self._timeout = timeout

        self._client = httpx.Client()

    def resolve_without_validation(self, did: str) -> dict:
        url = self._parse_web_did(did)

        try:
            response = self._client.get(url, timeout=self._timeout)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPError as e:
            raise DidWebResolverError(f'Error resolving DID {did}') from e


class AsyncDidWebResolver(_DidWebResolverBase, AsyncBaseResolver):
    def __init__(
        self,
        timeout: t.Optional[float] = None,
        cache: t.Optional['AsyncDidBaseCache'] = None,
    ) -> None:
        super().__init__(cache)

        self._timeout = timeout

        self._client = httpx.AsyncClient()

    async def resolve_without_validation(self, did: str) -> dict:
        url = self._parse_web_did(did)

        try:
            response = await self._client.get(url, timeout=self._timeout)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPError as e:
            raise DidWebResolverError(f'Error resolving DID {did}') from e



from atproto_client.models.app.bsky.actor import defs as AppBskyActorDefs
from atproto_client.models.app.bsky.actor import get_preferences as AppBskyActorGetPreferences
from atproto_client.models.app.bsky.actor import get_profile as AppBskyActorGetProfile
from atproto_client.models.app.bsky.actor import get_profiles as AppBskyActorGetProfiles
from atproto_client.models.app.bsky.actor import get_suggestions as AppBskyActorGetSuggestions
from atproto_client.models.app.bsky.actor import profile as AppBskyActorProfile
from atproto_client.models.app.bsky.actor import put_preferences as AppBskyActorPutPreferences
from atproto_client.models.app.bsky.actor import search_actors as AppBskyActorSearchActors
from atproto_client.models.app.bsky.actor import search_actors_typeahead as AppBskyActorSearchActorsTypeahead
from atproto_client.models.app.bsky.embed import external as AppBskyEmbedExternal
from atproto_client.models.app.bsky.embed import images as AppBskyEmbedImages
from atproto_client.models.app.bsky.embed import record as AppBskyEmbedRecord
from atproto_client.models.app.bsky.embed import record_with_media as AppBskyEmbedRecordWithMedia
from atproto_client.models.app.bsky.feed import defs as AppBskyFeedDefs
from atproto_client.models.app.bsky.feed import describe_feed_generator as AppBskyFeedDescribeFeedGenerator
from atproto_client.models.app.bsky.feed import generator as AppBskyFeedGenerator
from atproto_client.models.app.bsky.feed import get_actor_feeds as AppBskyFeedGetActorFeeds
from atproto_client.models.app.bsky.feed import get_actor_likes as AppBskyFeedGetActorLikes
from atproto_client.models.app.bsky.feed import get_author_feed as AppBskyFeedGetAuthorFeed
from atproto_client.models.app.bsky.feed import get_feed as AppBskyFeedGetFeed
from atproto_client.models.app.bsky.feed import get_feed_generator as AppBskyFeedGetFeedGenerator
from atproto_client.models.app.bsky.feed import get_feed_generators as AppBskyFeedGetFeedGenerators
from atproto_client.models.app.bsky.feed import get_feed_skeleton as AppBskyFeedGetFeedSkeleton
from atproto_client.models.app.bsky.feed import get_likes as AppBskyFeedGetLikes
from atproto_client.models.app.bsky.feed import get_list_feed as AppBskyFeedGetListFeed
from atproto_client.models.app.bsky.feed import get_post_thread as AppBskyFeedGetPostThread
from atproto_client.models.app.bsky.feed import get_posts as AppBskyFeedGetPosts
from atproto_client.models.app.bsky.feed import get_reposted_by as AppBskyFeedGetRepostedBy
from atproto_client.models.app.bsky.feed import get_suggested_feeds as AppBskyFeedGetSuggestedFeeds
from atproto_client.models.app.bsky.feed import get_timeline as AppBskyFeedGetTimeline
from atproto_client.models.app.bsky.feed import like as AppBskyFeedLike
from atproto_client.models.app.bsky.feed import post as AppBskyFeedPost
from atproto_client.models.app.bsky.feed import repost as AppBskyFeedRepost
from atproto_client.models.app.bsky.feed import search_posts as AppBskyFeedSearchPosts
from atproto_client.models.app.bsky.feed import threadgate as AppBskyFeedThreadgate
from atproto_client.models.app.bsky.graph import block as AppBskyGraphBlock
from atproto_client.models.app.bsky.graph import defs as AppBskyGraphDefs
from atproto_client.models.app.bsky.graph import follow as AppBskyGraphFollow
from atproto_client.models.app.bsky.graph import get_blocks as AppBskyGraphGetBlocks
from atproto_client.models.app.bsky.graph import get_followers as AppBskyGraphGetFollowers
from atproto_client.models.app.bsky.graph import get_follows as AppBskyGraphGetFollows
from atproto_client.models.app.bsky.graph import get_list as AppBskyGraphGetList
from atproto_client.models.app.bsky.graph import get_list_blocks as AppBskyGraphGetListBlocks
from atproto_client.models.app.bsky.graph import get_list_mutes as AppBskyGraphGetListMutes
from atproto_client.models.app.bsky.graph import get_lists as AppBskyGraphGetLists
from atproto_client.models.app.bsky.graph import get_mutes as AppBskyGraphGetMutes
from atproto_client.models.app.bsky.graph import get_relationships as AppBskyGraphGetRelationships
from atproto_client.models.app.bsky.graph import (
    get_suggested_follows_by_actor as AppBskyGraphGetSuggestedFollowsByActor,
)
from atproto_client.models.app.bsky.graph import list as AppBskyGraphList
from atproto_client.models.app.bsky.graph import listblock as AppBskyGraphListblock
from atproto_client.models.app.bsky.graph import listitem as AppBskyGraphListitem
from atproto_client.models.app.bsky.graph import mute_actor as AppBskyGraphMuteActor
from atproto_client.models.app.bsky.graph import mute_actor_list as AppBskyGraphMuteActorList
from atproto_client.models.app.bsky.graph import unmute_actor as AppBskyGraphUnmuteActor
from atproto_client.models.app.bsky.graph import unmute_actor_list as AppBskyGraphUnmuteActorList
from atproto_client.models.app.bsky.notification import get_unread_count as AppBskyNotificationGetUnreadCount
from atproto_client.models.app.bsky.notification import list_notifications as AppBskyNotificationListNotifications
from atproto_client.models.app.bsky.notification import register_push as AppBskyNotificationRegisterPush
from atproto_client.models.app.bsky.notification import update_seen as AppBskyNotificationUpdateSeen
from atproto_client.models.app.bsky.richtext import facet as AppBskyRichtextFacet
from atproto_client.models.app.bsky.unspecced import defs as AppBskyUnspeccedDefs
from atproto_client.models.app.bsky.unspecced import (
    get_popular_feed_generators as AppBskyUnspeccedGetPopularFeedGenerators,
)
from atproto_client.models.app.bsky.unspecced import get_tagged_suggestions as AppBskyUnspeccedGetTaggedSuggestions
from atproto_client.models.app.bsky.unspecced import search_actors_skeleton as AppBskyUnspeccedSearchActorsSkeleton
from atproto_client.models.app.bsky.unspecced import search_posts_skeleton as AppBskyUnspeccedSearchPostsSkeleton
from atproto_client.models.com.atproto.admin import (
    create_communication_template as ComAtprotoAdminCreateCommunicationTemplate,
)
from atproto_client.models.com.atproto.admin import defs as ComAtprotoAdminDefs
from atproto_client.models.com.atproto.admin import delete_account as ComAtprotoAdminDeleteAccount
from atproto_client.models.com.atproto.admin import (
    delete_communication_template as ComAtprotoAdminDeleteCommunicationTemplate,
)
from atproto_client.models.com.atproto.admin import disable_account_invites as ComAtprotoAdminDisableAccountInvites
from atproto_client.models.com.atproto.admin import disable_invite_codes as ComAtprotoAdminDisableInviteCodes
from atproto_client.models.com.atproto.admin import emit_moderation_event as ComAtprotoAdminEmitModerationEvent
from atproto_client.models.com.atproto.admin import enable_account_invites as ComAtprotoAdminEnableAccountInvites
from atproto_client.models.com.atproto.admin import get_account_info as ComAtprotoAdminGetAccountInfo
from atproto_client.models.com.atproto.admin import get_account_infos as ComAtprotoAdminGetAccountInfos
from atproto_client.models.com.atproto.admin import get_invite_codes as ComAtprotoAdminGetInviteCodes
from atproto_client.models.com.atproto.admin import get_moderation_event as ComAtprotoAdminGetModerationEvent
from atproto_client.models.com.atproto.admin import get_record as ComAtprotoAdminGetRecord
from atproto_client.models.com.atproto.admin import get_repo as ComAtprotoAdminGetRepo
from atproto_client.models.com.atproto.admin import get_subject_status as ComAtprotoAdminGetSubjectStatus
from atproto_client.models.com.atproto.admin import (
    list_communication_templates as ComAtprotoAdminListCommunicationTemplates,
)
from atproto_client.models.com.atproto.admin import query_moderation_events as ComAtprotoAdminQueryModerationEvents
from atproto_client.models.com.atproto.admin import query_moderation_statuses as ComAtprotoAdminQueryModerationStatuses
from atproto_client.models.com.atproto.admin import search_repos as ComAtprotoAdminSearchRepos
from atproto_client.models.com.atproto.admin import send_email as ComAtprotoAdminSendEmail
from atproto_client.models.com.atproto.admin import update_account_email as ComAtprotoAdminUpdateAccountEmail
from atproto_client.models.com.atproto.admin import update_account_handle as ComAtprotoAdminUpdateAccountHandle
from atproto_client.models.com.atproto.admin import (
    update_communication_template as ComAtprotoAdminUpdateCommunicationTemplate,
)
from atproto_client.models.com.atproto.admin import update_subject_status as ComAtprotoAdminUpdateSubjectStatus
from atproto_client.models.com.atproto.identity import resolve_handle as ComAtprotoIdentityResolveHandle
from atproto_client.models.com.atproto.identity import update_handle as ComAtprotoIdentityUpdateHandle
from atproto_client.models.com.atproto.label import defs as ComAtprotoLabelDefs
from atproto_client.models.com.atproto.label import query_labels as ComAtprotoLabelQueryLabels
from atproto_client.models.com.atproto.label import subscribe_labels as ComAtprotoLabelSubscribeLabels
from atproto_client.models.com.atproto.moderation import create_report as ComAtprotoModerationCreateReport
from atproto_client.models.com.atproto.moderation import defs as ComAtprotoModerationDefs
from atproto_client.models.com.atproto.repo import apply_writes as ComAtprotoRepoApplyWrites
from atproto_client.models.com.atproto.repo import create_record as ComAtprotoRepoCreateRecord
from atproto_client.models.com.atproto.repo import delete_record as ComAtprotoRepoDeleteRecord
from atproto_client.models.com.atproto.repo import describe_repo as ComAtprotoRepoDescribeRepo
from atproto_client.models.com.atproto.repo import get_record as ComAtprotoRepoGetRecord
from atproto_client.models.com.atproto.repo import list_records as ComAtprotoRepoListRecords
from atproto_client.models.com.atproto.repo import put_record as ComAtprotoRepoPutRecord
from atproto_client.models.com.atproto.repo import strong_ref as ComAtprotoRepoStrongRef
from atproto_client.models.com.atproto.repo import upload_blob as ComAtprotoRepoUploadBlob
from atproto_client.models.com.atproto.server import confirm_email as ComAtprotoServerConfirmEmail
from atproto_client.models.com.atproto.server import create_account as ComAtprotoServerCreateAccount
from atproto_client.models.com.atproto.server import create_app_password as ComAtprotoServerCreateAppPassword
from atproto_client.models.com.atproto.server import create_invite_code as ComAtprotoServerCreateInviteCode
from atproto_client.models.com.atproto.server import create_invite_codes as ComAtprotoServerCreateInviteCodes
from atproto_client.models.com.atproto.server import create_session as ComAtprotoServerCreateSession
from atproto_client.models.com.atproto.server import defs as ComAtprotoServerDefs
from atproto_client.models.com.atproto.server import delete_account as ComAtprotoServerDeleteAccount
from atproto_client.models.com.atproto.server import delete_session as ComAtprotoServerDeleteSession
from atproto_client.models.com.atproto.server import describe_server as ComAtprotoServerDescribeServer
from atproto_client.models.com.atproto.server import get_account_invite_codes as ComAtprotoServerGetAccountInviteCodes
from atproto_client.models.com.atproto.server import get_session as ComAtprotoServerGetSession
from atproto_client.models.com.atproto.server import list_app_passwords as ComAtprotoServerListAppPasswords
from atproto_client.models.com.atproto.server import refresh_session as ComAtprotoServerRefreshSession
from atproto_client.models.com.atproto.server import request_account_delete as ComAtprotoServerRequestAccountDelete
from atproto_client.models.com.atproto.server import (
    request_email_confirmation as ComAtprotoServerRequestEmailConfirmation,
)
from atproto_client.models.com.atproto.server import request_email_update as ComAtprotoServerRequestEmailUpdate
from atproto_client.models.com.atproto.server import request_password_reset as ComAtprotoServerRequestPasswordReset
from atproto_client.models.com.atproto.server import reserve_signing_key as ComAtprotoServerReserveSigningKey
from atproto_client.models.com.atproto.server import reset_password as ComAtprotoServerResetPassword
from atproto_client.models.com.atproto.server import revoke_app_password as ComAtprotoServerRevokeAppPassword
from atproto_client.models.com.atproto.server import update_email as ComAtprotoServerUpdateEmail
from atproto_client.models.com.atproto.sync import get_blob as ComAtprotoSyncGetBlob
from atproto_client.models.com.atproto.sync import get_blocks as ComAtprotoSyncGetBlocks
from atproto_client.models.com.atproto.sync import get_checkout as ComAtprotoSyncGetCheckout
from atproto_client.models.com.atproto.sync import get_head as ComAtprotoSyncGetHead
from atproto_client.models.com.atproto.sync import get_latest_commit as ComAtprotoSyncGetLatestCommit
from atproto_client.models.com.atproto.sync import get_record as ComAtprotoSyncGetRecord
from atproto_client.models.com.atproto.sync import get_repo as ComAtprotoSyncGetRepo
from atproto_client.models.com.atproto.sync import list_blobs as ComAtprotoSyncListBlobs
from atproto_client.models.com.atproto.sync import list_repos as ComAtprotoSyncListRepos
from atproto_client.models.com.atproto.sync import notify_of_update as ComAtprotoSyncNotifyOfUpdate
from atproto_client.models.com.atproto.sync import request_crawl as ComAtprotoSyncRequestCrawl
from atproto_client.models.com.atproto.sync import subscribe_repos as ComAtprotoSyncSubscribeRepos
from atproto_client.models.com.atproto.temp import check_signup_queue as ComAtprotoTempCheckSignupQueue
from atproto_client.models.com.atproto.temp import fetch_labels as ComAtprotoTempFetchLabels
from atproto_client.models.com.atproto.temp import import_repo as ComAtprotoTempImportRepo
from atproto_client.models.com.atproto.temp import push_blob as ComAtprotoTempPushBlob
from atproto_client.models.com.atproto.temp import request_phone_verification as ComAtprotoTempRequestPhoneVerification
from atproto_client.models.com.atproto.temp import transfer_account as ComAtprotoTempTransferAccount
from atproto_client.models.models_loader import load_models
from atproto_client.models.utils import (
    create_strong_ref,
    get_model_as_dict,
    get_model_as_json,
    get_or_create,
    is_record_type,
)


class _Ids:
    AppBskyActorDefs: str = 'app.bsky.actor.defs'
    AppBskyActorGetPreferences: str = 'app.bsky.actor.getPreferences'
    AppBskyActorGetProfile: str = 'app.bsky.actor.getProfile'
    AppBskyActorGetProfiles: str = 'app.bsky.actor.getProfiles'
    AppBskyActorGetSuggestions: str = 'app.bsky.actor.getSuggestions'
    AppBskyActorProfile: str = 'app.bsky.actor.profile'
    AppBskyActorPutPreferences: str = 'app.bsky.actor.putPreferences'
    AppBskyActorSearchActors: str = 'app.bsky.actor.searchActors'
    AppBskyActorSearchActorsTypeahead: str = 'app.bsky.actor.searchActorsTypeahead'
    AppBskyEmbedExternal: str = 'app.bsky.embed.external'
    AppBskyEmbedImages: str = 'app.bsky.embed.images'
    AppBskyEmbedRecord: str = 'app.bsky.embed.record'
    AppBskyEmbedRecordWithMedia: str = 'app.bsky.embed.recordWithMedia'
    AppBskyFeedDefs: str = 'app.bsky.feed.defs'
    AppBskyFeedDescribeFeedGenerator: str = 'app.bsky.feed.describeFeedGenerator'
    AppBskyFeedGenerator: str = 'app.bsky.feed.generator'
    AppBskyFeedGetActorFeeds: str = 'app.bsky.feed.getActorFeeds'
    AppBskyFeedGetActorLikes: str = 'app.bsky.feed.getActorLikes'
    AppBskyFeedGetAuthorFeed: str = 'app.bsky.feed.getAuthorFeed'
    AppBskyFeedGetFeed: str = 'app.bsky.feed.getFeed'
    AppBskyFeedGetFeedGenerator: str = 'app.bsky.feed.getFeedGenerator'
    AppBskyFeedGetFeedGenerators: str = 'app.bsky.feed.getFeedGenerators'
    AppBskyFeedGetFeedSkeleton: str = 'app.bsky.feed.getFeedSkeleton'
    AppBskyFeedGetLikes: str = 'app.bsky.feed.getLikes'
    AppBskyFeedGetListFeed: str = 'app.bsky.feed.getListFeed'
    AppBskyFeedGetPostThread: str = 'app.bsky.feed.getPostThread'
    AppBskyFeedGetPosts: str = 'app.bsky.feed.getPosts'
    AppBskyFeedGetRepostedBy: str = 'app.bsky.feed.getRepostedBy'
    AppBskyFeedGetSuggestedFeeds: str = 'app.bsky.feed.getSuggestedFeeds'
    AppBskyFeedGetTimeline: str = 'app.bsky.feed.getTimeline'
    AppBskyFeedLike: str = 'app.bsky.feed.like'
    AppBskyFeedPost: str = 'app.bsky.feed.post'
    AppBskyFeedRepost: str = 'app.bsky.feed.repost'
    AppBskyFeedSearchPosts: str = 'app.bsky.feed.searchPosts'
    AppBskyFeedThreadgate: str = 'app.bsky.feed.threadgate'
    AppBskyGraphBlock: str = 'app.bsky.graph.block'
    AppBskyGraphDefs: str = 'app.bsky.graph.defs'
    AppBskyGraphFollow: str = 'app.bsky.graph.follow'
    AppBskyGraphGetBlocks: str = 'app.bsky.graph.getBlocks'
    AppBskyGraphGetFollowers: str = 'app.bsky.graph.getFollowers'
    AppBskyGraphGetFollows: str = 'app.bsky.graph.getFollows'
    AppBskyGraphGetList: str = 'app.bsky.graph.getList'
    AppBskyGraphGetListBlocks: str = 'app.bsky.graph.getListBlocks'
    AppBskyGraphGetListMutes: str = 'app.bsky.graph.getListMutes'
    AppBskyGraphGetLists: str = 'app.bsky.graph.getLists'
    AppBskyGraphGetMutes: str = 'app.bsky.graph.getMutes'
    AppBskyGraphGetRelationships: str = 'app.bsky.graph.getRelationships'
    AppBskyGraphGetSuggestedFollowsByActor: str = 'app.bsky.graph.getSuggestedFollowsByActor'
    AppBskyGraphList: str = 'app.bsky.graph.list'
    AppBskyGraphListblock: str = 'app.bsky.graph.listblock'
    AppBskyGraphListitem: str = 'app.bsky.graph.listitem'
    AppBskyGraphMuteActor: str = 'app.bsky.graph.muteActor'
    AppBskyGraphMuteActorList: str = 'app.bsky.graph.muteActorList'
    AppBskyGraphUnmuteActor: str = 'app.bsky.graph.unmuteActor'
    AppBskyGraphUnmuteActorList: str = 'app.bsky.graph.unmuteActorList'
    AppBskyNotificationGetUnreadCount: str = 'app.bsky.notification.getUnreadCount'
    AppBskyNotificationListNotifications: str = 'app.bsky.notification.listNotifications'
    AppBskyNotificationRegisterPush: str = 'app.bsky.notification.registerPush'
    AppBskyNotificationUpdateSeen: str = 'app.bsky.notification.updateSeen'
    AppBskyRichtextFacet: str = 'app.bsky.richtext.facet'
    AppBskyUnspeccedDefs: str = 'app.bsky.unspecced.defs'
    AppBskyUnspeccedGetPopularFeedGenerators: str = 'app.bsky.unspecced.getPopularFeedGenerators'
    AppBskyUnspeccedGetTaggedSuggestions: str = 'app.bsky.unspecced.getTaggedSuggestions'
    AppBskyUnspeccedSearchActorsSkeleton: str = 'app.bsky.unspecced.searchActorsSkeleton'
    AppBskyUnspeccedSearchPostsSkeleton: str = 'app.bsky.unspecced.searchPostsSkeleton'
    ComAtprotoAdminCreateCommunicationTemplate: str = 'com.atproto.admin.createCommunicationTemplate'
    ComAtprotoAdminDefs: str = 'com.atproto.admin.defs'
    ComAtprotoAdminDeleteAccount: str = 'com.atproto.admin.deleteAccount'
    ComAtprotoAdminDeleteCommunicationTemplate: str = 'com.atproto.admin.deleteCommunicationTemplate'
    ComAtprotoAdminDisableAccountInvites: str = 'com.atproto.admin.disableAccountInvites'
    ComAtprotoAdminDisableInviteCodes: str = 'com.atproto.admin.disableInviteCodes'
    ComAtprotoAdminEmitModerationEvent: str = 'com.atproto.admin.emitModerationEvent'
    ComAtprotoAdminEnableAccountInvites: str = 'com.atproto.admin.enableAccountInvites'
    ComAtprotoAdminGetAccountInfo: str = 'com.atproto.admin.getAccountInfo'
    ComAtprotoAdminGetAccountInfos: str = 'com.atproto.admin.getAccountInfos'
    ComAtprotoAdminGetInviteCodes: str = 'com.atproto.admin.getInviteCodes'
    ComAtprotoAdminGetModerationEvent: str = 'com.atproto.admin.getModerationEvent'
    ComAtprotoAdminGetRecord: str = 'com.atproto.admin.getRecord'
    ComAtprotoAdminGetRepo: str = 'com.atproto.admin.getRepo'
    ComAtprotoAdminGetSubjectStatus: str = 'com.atproto.admin.getSubjectStatus'
    ComAtprotoAdminListCommunicationTemplates: str = 'com.atproto.admin.listCommunicationTemplates'
    ComAtprotoAdminQueryModerationEvents: str = 'com.atproto.admin.queryModerationEvents'
    ComAtprotoAdminQueryModerationStatuses: str = 'com.atproto.admin.queryModerationStatuses'
    ComAtprotoAdminSearchRepos: str = 'com.atproto.admin.searchRepos'
    ComAtprotoAdminSendEmail: str = 'com.atproto.admin.sendEmail'
    ComAtprotoAdminUpdateAccountEmail: str = 'com.atproto.admin.updateAccountEmail'
    ComAtprotoAdminUpdateAccountHandle: str = 'com.atproto.admin.updateAccountHandle'
    ComAtprotoAdminUpdateCommunicationTemplate: str = 'com.atproto.admin.updateCommunicationTemplate'
    ComAtprotoAdminUpdateSubjectStatus: str = 'com.atproto.admin.updateSubjectStatus'
    ComAtprotoIdentityResolveHandle: str = 'com.atproto.identity.resolveHandle'
    ComAtprotoIdentityUpdateHandle: str = 'com.atproto.identity.updateHandle'
    ComAtprotoLabelDefs: str = 'com.atproto.label.defs'
    ComAtprotoLabelQueryLabels: str = 'com.atproto.label.queryLabels'
    ComAtprotoLabelSubscribeLabels: str = 'com.atproto.label.subscribeLabels'
    ComAtprotoModerationCreateReport: str = 'com.atproto.moderation.createReport'
    ComAtprotoModerationDefs: str = 'com.atproto.moderation.defs'
    ComAtprotoRepoApplyWrites: str = 'com.atproto.repo.applyWrites'
    ComAtprotoRepoCreateRecord: str = 'com.atproto.repo.createRecord'
    ComAtprotoRepoDeleteRecord: str = 'com.atproto.repo.deleteRecord'
    ComAtprotoRepoDescribeRepo: str = 'com.atproto.repo.describeRepo'
    ComAtprotoRepoGetRecord: str = 'com.atproto.repo.getRecord'
    ComAtprotoRepoListRecords: str = 'com.atproto.repo.listRecords'
    ComAtprotoRepoPutRecord: str = 'com.atproto.repo.putRecord'
    ComAtprotoRepoStrongRef: str = 'com.atproto.repo.strongRef'
    ComAtprotoRepoUploadBlob: str = 'com.atproto.repo.uploadBlob'
    ComAtprotoServerConfirmEmail: str = 'com.atproto.server.confirmEmail'
    ComAtprotoServerCreateAccount: str = 'com.atproto.server.createAccount'
    ComAtprotoServerCreateAppPassword: str = 'com.atproto.server.createAppPassword'
    ComAtprotoServerCreateInviteCode: str = 'com.atproto.server.createInviteCode'
    ComAtprotoServerCreateInviteCodes: str = 'com.atproto.server.createInviteCodes'
    ComAtprotoServerCreateSession: str = 'com.atproto.server.createSession'
    ComAtprotoServerDefs: str = 'com.atproto.server.defs'
    ComAtprotoServerDeleteAccount: str = 'com.atproto.server.deleteAccount'
    ComAtprotoServerDeleteSession: str = 'com.atproto.server.deleteSession'
    ComAtprotoServerDescribeServer: str = 'com.atproto.server.describeServer'
    ComAtprotoServerGetAccountInviteCodes: str = 'com.atproto.server.getAccountInviteCodes'
    ComAtprotoServerGetSession: str = 'com.atproto.server.getSession'
    ComAtprotoServerListAppPasswords: str = 'com.atproto.server.listAppPasswords'
    ComAtprotoServerRefreshSession: str = 'com.atproto.server.refreshSession'
    ComAtprotoServerRequestAccountDelete: str = 'com.atproto.server.requestAccountDelete'
    ComAtprotoServerRequestEmailConfirmation: str = 'com.atproto.server.requestEmailConfirmation'
    ComAtprotoServerRequestEmailUpdate: str = 'com.atproto.server.requestEmailUpdate'
    ComAtprotoServerRequestPasswordReset: str = 'com.atproto.server.requestPasswordReset'
    ComAtprotoServerReserveSigningKey: str = 'com.atproto.server.reserveSigningKey'
    ComAtprotoServerResetPassword: str = 'com.atproto.server.resetPassword'
    ComAtprotoServerRevokeAppPassword: str = 'com.atproto.server.revokeAppPassword'
    ComAtprotoServerUpdateEmail: str = 'com.atproto.server.updateEmail'
    ComAtprotoSyncGetBlob: str = 'com.atproto.sync.getBlob'
    ComAtprotoSyncGetBlocks: str = 'com.atproto.sync.getBlocks'
    ComAtprotoSyncGetCheckout: str = 'com.atproto.sync.getCheckout'
    ComAtprotoSyncGetHead: str = 'com.atproto.sync.getHead'
    ComAtprotoSyncGetLatestCommit: str = 'com.atproto.sync.getLatestCommit'
    ComAtprotoSyncGetRecord: str = 'com.atproto.sync.getRecord'
    ComAtprotoSyncGetRepo: str = 'com.atproto.sync.getRepo'
    ComAtprotoSyncListBlobs: str = 'com.atproto.sync.listBlobs'
    ComAtprotoSyncListRepos: str = 'com.atproto.sync.listRepos'
    ComAtprotoSyncNotifyOfUpdate: str = 'com.atproto.sync.notifyOfUpdate'
    ComAtprotoSyncRequestCrawl: str = 'com.atproto.sync.requestCrawl'
    ComAtprotoSyncSubscribeRepos: str = 'com.atproto.sync.subscribeRepos'
    ComAtprotoTempCheckSignupQueue: str = 'com.atproto.temp.checkSignupQueue'
    ComAtprotoTempFetchLabels: str = 'com.atproto.temp.fetchLabels'
    ComAtprotoTempImportRepo: str = 'com.atproto.temp.importRepo'
    ComAtprotoTempPushBlob: str = 'com.atproto.temp.pushBlob'
    ComAtprotoTempRequestPhoneVerification: str = 'com.atproto.temp.requestPhoneVerification'
    ComAtprotoTempTransferAccount: str = 'com.atproto.temp.transferAccount'


ids = _Ids()
load_models()



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


from pydantic import Field

from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.requestEmailUpdate`."""

    token_required: bool = Field(alias='tokenRequired')  #: Token required.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownInputType, UnknownType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.createAccount`."""

    handle: str  #: Handle.
    did: t.Optional[str] = None  #: Did.
    email: t.Optional[str] = None  #: Email.
    invite_code: t.Optional[str] = Field(default=None, alias='inviteCode')  #: Invite code.
    password: t.Optional[str] = None  #: Password.
    plc_op: t.Optional['UnknownInputType'] = Field(default=None, alias='plcOp')  #: Plc op.
    recovery_key: t.Optional[str] = Field(default=None, alias='recoveryKey')  #: Recovery key.
    verification_code: t.Optional[str] = Field(default=None, alias='verificationCode')  #: Verification code.
    verification_phone: t.Optional[str] = Field(default=None, alias='verificationPhone')  #: Verification phone.


class DataDict(te.TypedDict):
    handle: str  #: Handle.
    did: te.NotRequired[t.Optional[str]]  #: Did.
    email: te.NotRequired[t.Optional[str]]  #: Email.
    invite_code: te.NotRequired[t.Optional[str]]  #: Invite code.
    password: te.NotRequired[t.Optional[str]]  #: Password.
    plc_op: te.NotRequired[t.Optional['UnknownInputType']]  #: Plc op.
    recovery_key: te.NotRequired[t.Optional[str]]  #: Recovery key.
    verification_code: te.NotRequired[t.Optional[str]]  #: Verification code.
    verification_phone: te.NotRequired[t.Optional[str]]  #: Verification phone.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.createAccount`."""

    access_jwt: str = Field(alias='accessJwt')  #: Access jwt.
    did: str  #: Did.
    handle: str  #: Handle.
    refresh_jwt: str = Field(alias='refreshJwt')  #: Refresh jwt.
    did_doc: t.Optional['UnknownType'] = Field(default=None, alias='didDoc')  #: Did doc.



import typing as t
from dataclasses import dataclass
from urllib.parse import urlparse

from atproto_crypto.did import get_did_key
from atproto_identity.did.atproto_data import AtprotoData
from pydantic import BaseModel, Field, ValidationError

_AT_URI_PREFIX = 'at://'
_AT_URI_PREFIX_LEN = len(_AT_URI_PREFIX)
_ATPROTO_KEY_ID = '#atproto'


@dataclass
class SigningKey:
    """Public signing key for the account."""

    type: str
    public_key_multibase: str


def _validate_url(url: str) -> t.Optional[str]:
    try:
        parsed_url = urlparse(url)
    except Exception:  # noqa: BLE001
        return None

    if parsed_url.scheme not in {'http', 'https'}:
        return None
    if parsed_url.hostname is None:
        return None

    return url


def is_valid_did_doc(did_doc: t.Union[dict, t.Any]) -> bool:
    """Return whether the given DID document is valid.

    Args:
        did_doc: The raw DID document.

    Returns:
        :obj:`bool`: Whether the given DID document is valid.
    """
    try:
        DidDocument.from_dict(did_doc)
        return True
    except ValidationError:
        return False


class VerificationMethod(BaseModel):
    """Verification method."""

    id: str
    type: str
    controller: str
    public_key_multibase: t.Optional[str] = Field(default=None, alias='publicKeyMultibase')


class Service(BaseModel):
    """Service."""

    id: str
    type: str
    service_endpoint: t.Union[str, dict] = Field(alias='serviceEndpoint')


class DidDocument(BaseModel):
    """DID document."""

    id: str
    also_known_as: t.Optional[t.List[str]] = Field(default=None, alias='alsoKnownAs')
    verification_method: t.Optional[t.List['VerificationMethod']] = Field(default=None, alias='verificationMethod')
    service: t.Optional[t.List['Service']] = None

    def get_did(self: 'DidDocument') -> str:
        """Return the DID of the given DID document.

        Returns:
            :obj:`str`: The DID of the given DID document.
        """
        return self.id

    def get_handle(self: 'DidDocument') -> t.Optional[str]:
        """Return the handle of the given DID document.

        Returns:
            :obj:`str`: The handle of the given DID document, or ``None`` if not found.
        """
        aka = self.also_known_as
        if not aka:
            return None

        for name in aka:
            if name.startswith(_AT_URI_PREFIX):
                return name[_AT_URI_PREFIX_LEN:]

        return None

    def get_service_endpoint(self: 'DidDocument', id_: str, type_: str) -> t.Optional[str]:
        """Return the service endpoint of the given DID document.

        Args:
            id_: The service ID.
            type_: The service type.

        Returns:
            :obj:`str`: The service endpoint of the given DID document, or ``None`` if not found.
        """
        did = self.get_did()

        services = self.service
        if not services:
            return None

        for service in services:
            if (service.id == id_ or service.id == f'{did}{id_}') and service.type == type_:
                return _validate_url(service.service_endpoint)

        return None

    def get_pds_endpoint(self: 'DidDocument') -> t.Optional[str]:
        """Return the personal data server endpoint of the given DID document.

        Returns:
            :obj:`str`: The personal data server endpoint of the given DID document, or ``None`` if not found.
        """
        return self.get_service_endpoint('#atproto_pds', 'AtprotoPersonalDataServer')

    def get_feed_gen_endpoint(self: 'DidDocument') -> t.Optional[str]:
        """Return the feed generator endpoint of the given DID document.

        Returns:
            :obj:`str`: The feed generator endpoint of the given DID document, or ``None`` if not found.
        """
        return self.get_service_endpoint('#bsky_fg', 'BskyFeedGenerator')

    def get_notif_endpoint(self: 'DidDocument') -> t.Optional[str]:
        """Return the notification endpoint of the given DID document.

        Returns:
            :obj:`str`: The notification endpoint of the given DID document, or ``None`` if not found.
        """
        return self.get_service_endpoint('#bsky_notif', 'BskyNotificationService')

    def get_signing_key(self: 'DidDocument') -> t.Optional['SigningKey']:
        """Return the signing key of the given DID document.

        Returns:
            :obj:`SigningKey`: The signing key of the given DID document, or ``None`` if not found.
        """
        did = self.get_did()

        keys = self.verification_method
        if not keys:
            return None

        for key in keys:
            if (key.id == _ATPROTO_KEY_ID or key.id == f'{did}{_ATPROTO_KEY_ID}') and key.public_key_multibase:
                return SigningKey(type=key.type, public_key_multibase=key.public_key_multibase)

        return None

    def get_did_key(self) -> t.Optional[str]:
        """Return the DID key of the given DID document.

        Returns:
            :obj:`str`: The DID key of the given DID document, or ``None`` if not found.
        """
        key = self.get_signing_key()
        if key is None:
            return None

        return get_did_key(key.type, key.public_key_multibase)

    def to_atproto_data(self) -> AtprotoData:
        """Return the AtprotoData of the given DID document.

        Returns:
            :obj:`AtprotoData`: The AtprotoData of the given DID document.
        """
        return AtprotoData(
            did=self.id,
            signing_key=self.get_did_key(),
            handle=self.get_handle(),
            pds=self.get_pds_endpoint(),
        )

    @classmethod
    def from_dict(cls, did_doc: t.Union[dict, t.Any]) -> 'DidDocument':
        """Parse a DID document.

        Args:
            did_doc: The raw DID document.

        Returns:
            :obj:`DidDocument`: The parsed DID document.
        """
        if hasattr(did_doc, 'to_dict'):
            return DidDocument(**did_doc.to_dict())

        return DidDocument(**did_doc)



import asyncio
import random
import socket
import threading
import time
import traceback
import typing as t
from copy import deepcopy
from urllib.parse import urlencode

from atproto_client.models import get_model_as_dict
from atproto_client.models.base import ParamsModelBase
from atproto_client.models.common import XrpcError
from atproto_core.exceptions import DAGCBORDecodingError
from websockets.client import connect as aconnect
from websockets.exceptions import (
    ConnectionClosedError,
    ConnectionClosedOK,
    InvalidHandshake,
    PayloadTooBig,
    ProtocolError,
)
from websockets.sync.client import connect

from atproto_firehose.exceptions import FirehoseDecodingError, FirehoseError
from atproto_firehose.models import ErrorFrame, Frame, MessageFrame

_BASE_WEBSOCKET_URI = 'wss://bsky.network/xrpc'
_MAX_MESSAGE_SIZE_BYTES = 1024 * 1024 * 5  # 5MB

OnMessageCallback = t.Callable[['MessageFrame'], None]
AsyncOnMessageCallback = t.Callable[['MessageFrame'], t.Coroutine[t.Any, t.Any, None]]

OnCallbackErrorCallback = t.Callable[[BaseException], None]
AsyncOnCallbackErrorCallback = t.Callable[[BaseException], t.Coroutine[t.Any, t.Any, None]]


if t.TYPE_CHECKING:
    from websockets.client import ClientConnection as SyncWebSocketClient
    from websockets.legacy.client import Connect as AsyncConnect


def _build_websocket_uri(
    method: str, base_uri: t.Optional[str] = None, params: t.Optional[t.Dict[str, t.Any]] = None
) -> str:
    if base_uri is None:
        base_uri = _BASE_WEBSOCKET_URI

    query_string = ''
    if params:
        query_string = f'?{urlencode(params)}'

    return f'{base_uri}/{method}{query_string}'


def _handle_frame_decoding_error(exception: Exception) -> None:
    if isinstance(exception, (DAGCBORDecodingError, FirehoseDecodingError)):
        # Ignore an invalid atproto_firehose frame that could not be properly decoded.
        # It's better to ignore one frame rather than stop the whole connection
        # or trap into an infinite loop of reconnections.
        return

    raise exception


def _handle_websocket_error_or_stop(exception: Exception) -> bool:
    """Return if the connection should be properly being closed or reraise exception."""
    if isinstance(exception, (ConnectionClosedOK,)):
        return True
    if isinstance(exception, (ConnectionClosedError, InvalidHandshake, PayloadTooBig, ProtocolError, socket.gaierror)):
        return False
    if isinstance(exception, FirehoseError):
        raise exception

    raise FirehoseError from exception


def _get_message_frame_from_bytes_or_raise(data: bytes) -> MessageFrame:
    frame = Frame.from_bytes(data)
    if isinstance(frame, ErrorFrame):
        raise FirehoseError(XrpcError(frame.body.error, frame.body.message))
    if isinstance(frame, MessageFrame):
        return frame
    raise FirehoseDecodingError('Unknown frame type')


class _WebsocketClientBase:
    def __init__(
        self,
        method: str,
        base_uri: t.Optional[str] = None,
        params: t.Optional[t.Dict[str, t.Any]] = None,
    ) -> None:
        self._method = method
        self._base_uri = base_uri
        self._params = params

        self._reconnect_no = 0
        self._max_reconnect_delay_sec = 64

    def update_params(self, params: t.Union[ParamsModelBase, t.Dict[str, t.Any]]) -> None:
        """Update params.

        Warning:
            If you are using `params` arg at the client start, you must care about keeping params up to date.
            Otherwise, your client will be rolled back to the previous state (cursor) on reconnecting.
        """
        if isinstance(params, ParamsModelBase):
            self._params = get_model_as_dict(params)
        else:
            self._params = deepcopy(params)

    @property
    def _websocket_uri(self) -> str:
        # the user should care about updated params by himself
        return _build_websocket_uri(self._method, self._base_uri, self._params)

    def _get_client(self) -> 'SyncWebSocketClient':
        return connect(self._websocket_uri, max_size=_MAX_MESSAGE_SIZE_BYTES)

    def _get_async_client(self) -> 'AsyncConnect':
        # FIXME(DXsmiley): I've noticed that the close operation often takes the entire timeout for some reason
        #   By default, this is 10 seconds, which is pretty long.
        #   Maybe shorten it?
        return aconnect(self._websocket_uri, max_size=_MAX_MESSAGE_SIZE_BYTES)

    def _get_reconnection_delay(self) -> int:
        base_sec = 2**self._reconnect_no
        rand_sec = random.uniform(-0.5, 0.5)  # noqa: S311

        return min(base_sec, self._max_reconnect_delay_sec) + rand_sec


class _WebsocketClient(_WebsocketClientBase):
    def __init__(
        self, method: str, base_uri: t.Optional[str] = None, params: t.Optional[t.Dict[str, t.Any]] = None
    ) -> None:
        super().__init__(method, base_uri, params)

        # TODO(DXsmiley): Not sure if this should be a Lock or not, the async is using an Event now
        self._stop_lock = threading.Lock()

        self._on_message_callback: t.Optional[OnMessageCallback] = None
        self._on_callback_error_callback: t.Optional[OnCallbackErrorCallback] = None

    def _process_message_frame(self, frame: 'MessageFrame') -> None:
        try:
            if self._on_message_callback is not None:
                self._on_message_callback(frame)
        except Exception as e:  # noqa: BLE001
            if self._on_callback_error_callback:
                try:
                    self._on_callback_error_callback(e)
                except:  # noqa
                    traceback.print_exc()
            else:
                traceback.print_exc()

    def start(
        self,
        on_message_callback: OnMessageCallback,
        on_callback_error_callback: t.Optional[OnCallbackErrorCallback] = None,
    ) -> None:
        """Subscribe to Firehose and start client.

        Args:
            on_message_callback: Callback that will be called on the new Firehose message.
            on_callback_error_callback: Callback that will be called if the `on_message_callback` raised an exception.

        Returns:
            :obj:`None`
        """
        self._on_message_callback = on_message_callback
        self._on_callback_error_callback = on_callback_error_callback

        while not self._stop_lock.locked():
            try:
                if self._reconnect_no != 0:
                    time.sleep(self._get_reconnection_delay())

                with self._get_client() as client:
                    self._reconnect_no = 0

                    while not self._stop_lock.locked():
                        raw_frame = client.recv()
                        if isinstance(raw_frame, str):
                            # skip text frames (should not be occurred)
                            continue

                        try:
                            frame = _get_message_frame_from_bytes_or_raise(raw_frame)
                            self._process_message_frame(frame)
                        except Exception as e:  # noqa: BLE001
                            _handle_frame_decoding_error(e)
            except Exception as e:  # noqa: BLE001
                self._reconnect_no += 1

                should_stop = _handle_websocket_error_or_stop(e)
                if should_stop:
                    break

        if self._stop_lock.locked():
            self._stop_lock.release()

    def stop(self) -> None:
        """Unsubscribe and stop the Firehose client.

        Returns:
            :obj:`None`
        """
        if not self._stop_lock.locked():
            self._stop_lock.acquire()


class _AsyncWebsocketClient(_WebsocketClientBase):
    def __init__(
        self, method: str, base_uri: t.Optional[str] = None, params: t.Optional[t.Dict[str, t.Any]] = None
    ) -> None:
        super().__init__(method, base_uri, params)

        self._stop_event = asyncio.Event()

        self._on_message_callback: t.Optional[AsyncOnMessageCallback] = None
        self._on_callback_error_callback: t.Optional[AsyncOnCallbackErrorCallback] = None

    async def _process_message_frame(self, frame: 'MessageFrame') -> None:
        try:
            if self._on_message_callback is not None:
                await self._on_message_callback(frame)
        except Exception as e:  # noqa: BLE001
            if self._on_callback_error_callback:
                try:
                    await self._on_callback_error_callback(e)
                except:  # noqa
                    traceback.print_exc()
            else:
                traceback.print_exc()

    async def start(
        self,
        on_message_callback: AsyncOnMessageCallback,
        on_callback_error_callback: t.Optional[AsyncOnCallbackErrorCallback] = None,
    ) -> None:
        """Subscribe to Firehose and start client.

        Args:
            on_message_callback: Callback that will be called on the new Firehose message.
            on_callback_error_callback: Callback that will be called if the `on_message_callback` raised an exception.

        Returns:
            :obj:`None`
        """
        self._on_message_callback = on_message_callback
        self._on_callback_error_callback = on_callback_error_callback

        while not self._stop_event.is_set():
            try:
                if self._reconnect_no != 0:
                    await asyncio.sleep(self._get_reconnection_delay())

                async with self._get_async_client() as client:
                    self._reconnect_no = 0

                    while not self._stop_event.is_set():
                        raw_frame = await client.recv()
                        if isinstance(raw_frame, str):
                            # skip text frames (should not be occurred)
                            continue

                        try:
                            frame = _get_message_frame_from_bytes_or_raise(raw_frame)
                            await self._process_message_frame(frame)
                        except Exception as e:  # noqa: BLE001
                            _handle_frame_decoding_error(e)

            except Exception as e:  # noqa: BLE001
                self._reconnect_no += 1

                should_stop = _handle_websocket_error_or_stop(e)
                if should_stop:
                    break

    async def stop(self) -> None:
        """Unsubscribe and stop the Firehose client.

        Returns:
            :obj:`None`
        """
        self._stop_event.set()


FirehoseClient = _WebsocketClient
AsyncFirehoseClient = _AsyncWebsocketClient



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.feed.getLikes`."""

    uri: str  #: Uri.
    cid: t.Optional[str] = None  #: Cid.
    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.


class ParamsDict(te.TypedDict):
    uri: str  #: Uri.
    cid: te.NotRequired[t.Optional[str]]  #: Cid.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.feed.getLikes`."""

    likes: t.List['models.AppBskyFeedGetLikes.Like']  #: Likes.
    uri: str  #: Uri.
    cid: t.Optional[str] = None  #: Cid.
    cursor: t.Optional[str] = None  #: Cursor.


class Like(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.getLikes`."""

    actor: 'models.AppBskyActorDefs.ProfileView'  #: Actor.
    created_at: str = Field(alias='createdAt')  #: Created at.
    indexed_at: str = Field(alias='indexedAt')  #: Indexed at.

    py_type: te.Literal['app.bsky.feed.getLikes#like'] = Field(
        default='app.bsky.feed.getLikes#like', alias='$type', frozen=True
    )



import libipld


def multibase_to_bytes(data: str) -> bytes:
    """Decode multibase.

    Args:
        data: Multibase encoded data.

    Returns:
        :obj:`bytes`: Decoded data.
    """
    _, data = libipld.decode_multibase(data)
    return data


def bytes_to_multibase(encoding: str, data: bytes) -> str:
    """Encode multibase.

    Note:
        Multibase table is available here: https://github.com/multiformats/multibase/blob/master/multibase.csv

    Args:
        encoding: Encoding (the character from the table).
        data: Data.

    Returns:
        :obj:`str`: Multibase encoded data.
    """
    return libipld.encode_multibase(encoding, data)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class InviteCode(base.ModelBase):
    """Definition model for :obj:`com.atproto.server.defs`."""

    available: int  #: Available.
    code: str  #: Code.
    created_at: str = Field(alias='createdAt')  #: Created at.
    created_by: str = Field(alias='createdBy')  #: Created by.
    disabled: bool  #: Disabled.
    for_account: str = Field(alias='forAccount')  #: For account.
    uses: t.List['models.ComAtprotoServerDefs.InviteCodeUse']  #: Uses.

    py_type: te.Literal['com.atproto.server.defs#inviteCode'] = Field(
        default='com.atproto.server.defs#inviteCode', alias='$type', frozen=True
    )


class InviteCodeUse(base.ModelBase):
    """Definition model for :obj:`com.atproto.server.defs`."""

    used_at: str = Field(alias='usedAt')  #: Used at.
    used_by: str = Field(alias='usedBy')  #: Used by.

    py_type: te.Literal['com.atproto.server.defs#inviteCodeUse'] = Field(
        default='com.atproto.server.defs#inviteCodeUse', alias='$type', frozen=True
    )



import re
import typing as t
import urllib.parse as urlparse
from urllib.parse import urlencode

from atproto_core.exceptions import InvalidAtUriError

# ref: https://github.com/bluesky-social/atproto/blob/8c19ce991a766fd9cff5023160853ab1cb106f21/packages/uri/src/index.ts#LL5C38-L5C38
_ATP_URI_REGEX = r'^(at:\/\/)?((?:did:[a-z0-9:%-]+)|(?:[a-z0-9][a-z0-9.:-]*))(\/[^?#\s]*)?(\?[^#\s]+)?(#[^\s]+)?$'


class AtUri:
    """ATP URI Scheme.

    Examples:
        Repository: at://alice.host.com

        Repository: at://did:plc:bv6ggog3tya2z3vxsub7hnal

        Collection: at://alice.host.com/io.example.song

        Record: at://alice.host.com/io.example.song/3yI5-c1z-cc2p-1a

        Record Field: at://alice.host.com/io.example.song/3yI5-c1z-cc2p-1a#/title
    """

    def __init__(self, host: str, pathname: str, hash_: str, search_params: t.List[t.Tuple[str, t.Any]]) -> None:
        self.host = host
        self.pathname = pathname
        self.hash = hash_
        self.search_params = search_params

    @property
    def protocol(self) -> str:
        """Get protocol."""
        return 'at://'

    @property
    def origin(self) -> str:
        """Get origin."""
        return f'at://{self.host}'

    @property
    def hostname(self) -> str:
        """Get hostname."""
        return self.host

    @property
    def collection(self) -> str:
        """Get collection name."""
        for part in self.pathname.split('/'):
            if part:
                return part

        return ''

    @property
    def rkey(self) -> str:
        """Get record key (rkey)."""
        parts = [p for p in self.pathname.split('/') if p]
        if len(parts) > 1:
            return parts[1]

        return ''

    @property
    def http(self) -> str:
        """Convert instance to HTTP URI."""
        return str(self)

    @classmethod
    def from_str(cls, uri: str) -> 'AtUri':
        """Create `AtUri` instance from URI."""
        groups = re.findall(_ATP_URI_REGEX, uri, re.IGNORECASE)
        if not groups:
            raise InvalidAtUriError

        group = groups[0]
        return cls(host=group[1], pathname=group[2], hash_=group[4], search_params=urlparse.parse_qsl(group[3]))

    def __str__(self) -> str:
        path = self.pathname or '/'
        if not path.startswith('/'):
            path = f'/{path}'

        query = urlencode(self.search_params)
        if query:
            query = f'?{query}'

        hash_ = self.hash
        if hash_ and not hash_.startswith('#'):
            hash_ = f'#{hash_}'

        return f'at://{self.host}{path}{query}{hash_}'

    def __hash__(self) -> int:
        return hash(str(self))

    def __eq__(self, other: t.Any) -> bool:
        if isinstance(other, AtUri):
            return hash(self) == hash(other)

        return False



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.getSession`."""

    did: str  #: Did.
    handle: str  #: Handle.
    did_doc: t.Optional['UnknownType'] = Field(default=None, alias='didDoc')  #: Did doc.
    email: t.Optional[str] = None  #: Email.
    email_confirmed: t.Optional[bool] = Field(default=None, alias='emailConfirmed')  #: Email confirmed.



import typing as t

import dns.asyncresolver
import dns.resolver
import httpx
from dns.exception import DNSException

from atproto_identity.exceptions import DidNotFoundError

_ATPROTO_SUBDOMAIN = '_atproto'
_ATPROTO_RECORD_TYPE = 'TXT'
_ATPROTO_TXT_PREFIX = 'did='
_ATPROTO_TXT_PREFIX_LEN = len(_ATPROTO_TXT_PREFIX)
_ATPROTO_WELL_KNOWN_PATH = '/.well-known/atproto-did'
_ATPROTO_DID_PREFIX = 'did:'


class _HandleResolverBase:
    @staticmethod
    def _find_text_record(answers: dns.resolver.Answer) -> t.Optional[str]:
        for answer in answers:
            for item in answer.strings:
                value = item.decode('UTF-8')
                if value.startswith(_ATPROTO_TXT_PREFIX):
                    return value[_ATPROTO_TXT_PREFIX_LEN:]

        return None

    @staticmethod
    def _get_qname(handle: str) -> str:
        return f'{_ATPROTO_SUBDOMAIN}.{handle}'

    @staticmethod
    def _get_did_from_text_response(text: str) -> t.Optional[str]:
        lines = text.splitlines()
        if not lines:
            return None

        first_line = lines[0].strip()
        if first_line.startswith(_ATPROTO_DID_PREFIX):
            return first_line

        return None


class HandleResolver(_HandleResolverBase):
    """Handle Resolver.

    Args:
        timeout: Request timeout.
        backup_nameservers: Backup nameservers (for DNS resolve).
    """

    def __init__(
        self,
        timeout: t.Optional[float] = None,
        backup_nameservers: t.Optional[t.List[str]] = None,
    ) -> None:
        self._timeout = timeout
        self._backup_nameservers = backup_nameservers  # TODO(MarshalX): implement

        self._dns_resolver = dns.resolver
        self._http_client = httpx.Client()

    def resolve(self, handle: str) -> t.Optional[str]:
        """Resolve handle to DID.

        Uses DNS and HTTP to resolve handle to DID. The first successful result will be returned.

        Resolve order: DNS -> HTTP.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID or ``None`` if handle not found.
        """
        dns_resolve = self.resolve_dns(handle)
        if dns_resolve:
            return dns_resolve

        http_resolve = self.resolve_http(handle)
        if http_resolve:
            return http_resolve

        # TODO(MarshalX): add resolve DNS backup nameservers
        return None

    def ensure_resolve(self, handle: str) -> str:
        """Ensure handle is resolved to DID.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID.

        Raises:
            :obj:`DidNotFoundError`: Handle not found.
        """
        did = self.resolve(handle)
        if not did:
            raise DidNotFoundError(f'Unable to resolve handle: {handle}')

        return did

    def resolve_dns(self, handle: str) -> t.Optional[str]:
        """Resolve handle to DID using DNS.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID or ``None`` if handle not found.
        """
        try:
            answers = self._dns_resolver.resolve(self._get_qname(handle), _ATPROTO_RECORD_TYPE, lifetime=self._timeout)
        except DNSException:
            return None

        return self._find_text_record(answers)

    def resolve_http(self, handle: str) -> t.Optional[str]:
        """Resolve handle to DID using HTTP.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID or ``None`` if handle not found.
        """
        try:
            response = self._http_client.get(f'https://{handle}{_ATPROTO_WELL_KNOWN_PATH}', timeout=self._timeout)
            response.raise_for_status()
            return self._get_did_from_text_response(response.text)
        except httpx.HTTPError:
            return None


class AsyncHandleResolver(_HandleResolverBase):
    """Asynchronous Handle Resolver.

    Args:
        timeout: Request timeout.
        backup_nameservers: Backup nameservers (for DNS resolve).
    """

    def __init__(
        self,
        timeout: t.Optional[float] = None,
        backup_nameservers: t.Optional[t.List[str]] = None,
    ) -> None:
        self._timeout = timeout
        self._backup_nameservers = backup_nameservers  # TODO(MarshalX): implement

        self._dns_resolver = dns.asyncresolver
        self._http_client = httpx.AsyncClient()

    async def resolve(self, handle: str) -> t.Optional[str]:
        """Resolve handle to DID.

        Uses DNS and HTTP to resolve handle to DID. The first successful result will be returned.

        Resolve order: DNS -> HTTP.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID or ``None`` if handle not found.
        """
        dns_resolve = await self.resolve_dns(handle)
        if dns_resolve:
            return dns_resolve

        http_resolve = await self.resolve_http(handle)
        if http_resolve:
            return http_resolve

        # TODO(MarshalX): add resolve DNS backup nameservers
        return None

    async def ensure_resolve(self, handle: str) -> str:
        """Ensure handle is resolved to DID.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID.

        Raises:
            :obj:`DidNotFoundError`: Handle not found.
        """
        did = await self.resolve(handle)
        if not did:
            raise DidNotFoundError(f'Unable to resolve handle: {handle}')

        return did

    async def resolve_dns(self, handle: str) -> t.Optional[str]:
        """Resolve handle to DID using DNS.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID or ``None`` if handle not found.
        """
        try:
            answers = await self._dns_resolver.resolve(
                self._get_qname(handle), _ATPROTO_RECORD_TYPE, lifetime=self._timeout
            )
        except DNSException:
            return None

        return self._find_text_record(answers)

    async def resolve_http(self, handle: str) -> t.Optional[str]:
        """Resolve handle to DID using HTTP.

        Args:
            handle: Handle.

        Returns:
            :obj:`str`: DID or ``None`` if handle not found.
        """
        try:
            response = await self._http_client.get(f'https://{handle}{_ATPROTO_WELL_KNOWN_PATH}', timeout=self._timeout)
            response.raise_for_status()
            return self._get_did_from_text_response(response.text)
        except httpx.HTTPError:
            return None



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownInputType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.temp.transferAccount`."""

    did: str  #: Did.
    handle: str  #: Handle.
    plc_op: 'UnknownInputType' = Field(alias='plcOp')  #: Plc op.


class DataDict(te.TypedDict):
    did: str  #: Did.
    handle: str  #: Handle.
    plc_op: 'UnknownInputType'  #: Plc op.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.temp.transferAccount`."""

    access_jwt: str = Field(alias='accessJwt')  #: Access jwt.
    did: str  #: Did.
    handle: str  #: Handle.
    refresh_jwt: str = Field(alias='refreshJwt')  #: Refresh jwt.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.repo.applyWrites`."""

    repo: str  #: The handle or DID of the repo.
    writes: t.List[
        te.Annotated[
            t.Union[
                'models.ComAtprotoRepoApplyWrites.Create',
                'models.ComAtprotoRepoApplyWrites.Update',
                'models.ComAtprotoRepoApplyWrites.Delete',
            ],
            Field(discriminator='py_type'),
        ]
    ]  #: Writes.
    swap_commit: t.Optional[str] = Field(default=None, alias='swapCommit')  #: Swap commit.
    validate_: t.Optional[bool] = Field(default=True, alias='validate')  #: Flag for validating the records.


class DataDict(te.TypedDict):
    repo: str  #: The handle or DID of the repo.
    writes: t.List[
        te.Annotated[
            t.Union[
                'models.ComAtprotoRepoApplyWrites.Create',
                'models.ComAtprotoRepoApplyWrites.Update',
                'models.ComAtprotoRepoApplyWrites.Delete',
            ],
            Field(discriminator='py_type'),
        ]
    ]  #: Writes.
    swap_commit: te.NotRequired[t.Optional[str]]  #: Swap commit.
    validate: te.NotRequired[t.Optional[bool]]  #: Flag for validating the records.


class Create(base.ModelBase):
    """Definition model for :obj:`com.atproto.repo.applyWrites`. Create a new record."""

    collection: str  #: Collection.
    value: 'UnknownType'  #: Value.
    rkey: t.Optional[str] = Field(default=None, max_length=15)  #: Rkey.

    py_type: te.Literal['com.atproto.repo.applyWrites#create'] = Field(
        default='com.atproto.repo.applyWrites#create', alias='$type', frozen=True
    )


class Update(base.ModelBase):
    """Definition model for :obj:`com.atproto.repo.applyWrites`. Update an existing record."""

    collection: str  #: Collection.
    rkey: str  #: Rkey.
    value: 'UnknownType'  #: Value.

    py_type: te.Literal['com.atproto.repo.applyWrites#update'] = Field(
        default='com.atproto.repo.applyWrites#update', alias='$type', frozen=True
    )


class Delete(base.ModelBase):
    """Definition model for :obj:`com.atproto.repo.applyWrites`. Delete an existing record."""

    collection: str  #: Collection.
    rkey: str  #: Rkey.

    py_type: te.Literal['com.atproto.repo.applyWrites#delete'] = Field(
        default='com.atproto.repo.applyWrites#delete', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te
from pydantic import Field

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.createAppPassword`."""

    name: str  #: Name.


class DataDict(te.TypedDict):
    name: str  #: Name.


class AppPassword(base.ModelBase):
    """Definition model for :obj:`com.atproto.server.createAppPassword`."""

    created_at: str = Field(alias='createdAt')  #: Created at.
    name: str  #: Name.
    password: str  #: Password.

    py_type: te.Literal['com.atproto.server.createAppPassword#appPassword'] = Field(
        default='com.atproto.server.createAppPassword#appPassword', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te

from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.actor.getProfile`."""

    actor: str  #: Actor.


class ParamsDict(te.TypedDict):
    actor: str  #: Actor.



# well, this is hardcoded by human and must be updated manually.
# rly don't want to create some kind of dynamic types.
# and even don't have time to do this.
# in original code, TypeScript Omit<> is used for args instead of coping
# this approach provides more clean usage on the user side with type hints

RECORD_CREATE_RESPONSE_MODEL_TEMPLATE = """
class CreateRecordResponse(base.SugarResponseModelBase):
    \"""Create record response for :obj:`models.{record_import}.Record`.\"""

    uri: str  #: The URI of the record.
    cid: str  #: The CID of the record.
"""

RECORD_CREATE_METHOD_TEMPLATE = """
    {d}def create(
        self,
        repo: str,
        record: 'models.{record_import}.Record',
        rkey: t.Optional[str] = None,
        swap_commit: t.Optional[str] = None,
        validate: t.Optional[bool] = True,
        **kwargs: t.Any,
    ) -> 'models.{record_import}.CreateRecordResponse':
        \"""Create a new record.

        Args:
            repo: The repository (DID).
            record: The record.
            rkey: The record key (TID).
            swap_commit: The swap commit.
            validate: Whether to validate the record.
            **kwargs: Arbitrary arguments to HTTP request.

        Returns:
            :obj:`models.{record_import}.CreateRecordResponse`: Create record response.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        \"""
        data_model = models.ComAtprotoRepoCreateRecord.Data(
            collection='{collection}',
            repo=repo,
            record=record,
            rkey=rkey,
            swapCommit=swap_commit,
            validate=validate,
        )
        response = {c}self._client.invoke_procedure(
            'com.atproto.repo.createRecord',
            data=data_model,
            input_encoding='application/json',
            output_encoding='application/json',
            **kwargs,
        )
        response_model = get_response_model(response, models.ComAtprotoRepoCreateRecord.Response)
        return models.{record_import}.CreateRecordResponse(uri=response_model.uri, cid=response_model.cid)
"""

RECORD_GET_RESPONSE_MODEL_TEMPLATE = """
class GetRecordResponse(base.SugarResponseModelBase):
    \"""Get record response for :obj:`models.{record_import}.Record`.\"""

    uri: str  #: The URI of the record.
    value: 'models.{record_import}.Record'  #: The record.
    cid: t.Optional[str] = None  #: The CID of the record.
"""

RECORD_GET_METHOD_TEMPLATE = """
    {d}def get(
        self, repo: str, rkey: str, cid: t.Optional[str] = None, **kwargs: t.Any
    ) -> 'models.{record_import}.GetRecordResponse':
        \"""Get a record.

        Args:
            repo: The repository (DID).
            rkey: The record key (TID).
            cid: The CID of the record.
            **kwargs: Arbitrary arguments to HTTP request.

        Returns:
            :obj:`models.{record_import}.GetRecordResponse`: Get record response.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        \"""
        params_model = models.ComAtprotoRepoGetRecord.Params(
            collection='{collection}',
            repo=repo,
            rkey=rkey,
            cid=cid
        )
        response = {c}self._client.invoke_query(
            'com.atproto.repo.getRecord', params=params_model, output_encoding='application/json', **kwargs
        )
        response_model = get_response_model(response, models.ComAtprotoRepoGetRecord.Response)
        return models.{record_import}.GetRecordResponse(
            uri=response_model.uri,
            cid=response_model.cid,
            value=t.cast('models.{record_import}.Record', response_model.value),
        )
"""

RECORD_LIST_RESPONSE_MODEL_TEMPLATE = """
class ListRecordsResponse(base.SugarResponseModelBase):
    \"""List records response for :obj:`models.{record_import}.Record`.\"""

    records: t.Dict[str, 'models.{record_import}.Record']  #: Map of URIs to records.
    cursor: t.Optional[str] = None  #: Next page cursor.
"""

RECORD_LIST_METHOD_TEMPLATE = """
    {d}def list(
        self,
        repo: str,
        cursor: t.Optional[str] = None,
        limit: t.Optional[int] = None,
        reverse: t.Optional[bool] = None,
        **kwargs: t.Any,
    ) -> 'models.{record_import}.ListRecordsResponse':
        \"""List a range of records in a collection.

        Args:
            repo: The repository (DID).
            cursor: The cursor.
            limit: The limit.
            reverse: Whether to reverse the order.
            **kwargs: Arbitrary arguments to HTTP request.

        Returns:
            :obj:`models.{record_import}.ListRecordsResponse`: List records response.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        \"""
        params_model = models.ComAtprotoRepoListRecords.Params(
            collection='{collection}',
            repo=repo,
            cursor=cursor,
            limit=limit,
            reverse=reverse,
        )
        response = {c}self._client.invoke_query(
            'com.atproto.repo.listRecords', params=params_model, output_encoding='application/json', **kwargs
        )
        response_model = get_response_model(response, models.ComAtprotoRepoListRecords.Response)
        return models.{record_import}.ListRecordsResponse(
            records={{
                record.uri: t.cast('models.{record_import}.Record', record.value)
                for record in response_model.records
            }},
            cursor=response_model.cursor,
        )
"""

RECORD_DELETE_METHOD_TEMPLATE = """
    {d}def delete(
        self,
        repo: str,
        rkey: str,
        swap_commit: t.Optional[str] = None,
        swap_record: t.Optional[str] = None,
        **kwargs: t.Any,
    ) -> bool:
        \"""Delete a record, or ensure it doesn't exist.

        Args:
            repo: The repository (DID).
            rkey: The record key (TID).
            swap_commit: The swap commit.
            swap_record: The swap record.
            **kwargs: Arbitrary arguments to HTTP request.

        Returns:
            :obj:`bool`: Success status.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        \"""
        data_model = models.ComAtprotoRepoDeleteRecord.Data(
            collection='{collection}',
            repo=repo,
            rkey=rkey,
            swapCommit=swap_commit,
            swapRecord=swap_record,
        )
        response = {c}self._client.invoke_procedure(
            'com.atproto.repo.deleteRecord', data=data_model, input_encoding='application/json', **kwargs
        )
        return get_response_model(response, bool)
"""



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class PostView(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    author: 'models.AppBskyActorDefs.ProfileViewBasic'  #: Author.
    cid: str  #: Cid.
    indexed_at: str = Field(alias='indexedAt')  #: Indexed at.
    record: 'UnknownType'  #: Record.
    uri: str  #: Uri.
    embed: t.Optional[
        te.Annotated[
            t.Union[
                'models.AppBskyEmbedImages.View',
                'models.AppBskyEmbedExternal.View',
                'models.AppBskyEmbedRecord.View',
                'models.AppBskyEmbedRecordWithMedia.View',
            ],
            Field(default=None, discriminator='py_type'),
        ]
    ] = None  #: Embed.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    like_count: t.Optional[int] = Field(default=None, alias='likeCount')  #: Like count.
    reply_count: t.Optional[int] = Field(default=None, alias='replyCount')  #: Reply count.
    repost_count: t.Optional[int] = Field(default=None, alias='repostCount')  #: Repost count.
    threadgate: t.Optional['models.AppBskyFeedDefs.ThreadgateView'] = None  #: Threadgate.
    viewer: t.Optional['models.AppBskyFeedDefs.ViewerState'] = None  #: Viewer.

    py_type: te.Literal['app.bsky.feed.defs#postView'] = Field(
        default='app.bsky.feed.defs#postView', alias='$type', frozen=True
    )


class ViewerState(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    like: t.Optional[str] = None  #: Like.
    reply_disabled: t.Optional[bool] = Field(default=None, alias='replyDisabled')  #: Reply disabled.
    repost: t.Optional[str] = None  #: Repost.

    py_type: te.Literal['app.bsky.feed.defs#viewerState'] = Field(
        default='app.bsky.feed.defs#viewerState', alias='$type', frozen=True
    )


class FeedViewPost(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    post: 'models.AppBskyFeedDefs.PostView'  #: Post.
    reason: t.Optional[
        te.Annotated[t.Union['models.AppBskyFeedDefs.ReasonRepost'], Field(default=None, discriminator='py_type')]
    ] = None  #: Reason.
    reply: t.Optional['models.AppBskyFeedDefs.ReplyRef'] = None  #: Reply.

    py_type: te.Literal['app.bsky.feed.defs#feedViewPost'] = Field(
        default='app.bsky.feed.defs#feedViewPost', alias='$type', frozen=True
    )


class ReplyRef(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    parent: te.Annotated[
        t.Union[
            'models.AppBskyFeedDefs.PostView',
            'models.AppBskyFeedDefs.NotFoundPost',
            'models.AppBskyFeedDefs.BlockedPost',
        ],
        Field(discriminator='py_type'),
    ]  #: Parent.
    root: te.Annotated[
        t.Union[
            'models.AppBskyFeedDefs.PostView',
            'models.AppBskyFeedDefs.NotFoundPost',
            'models.AppBskyFeedDefs.BlockedPost',
        ],
        Field(discriminator='py_type'),
    ]  #: Root.

    py_type: te.Literal['app.bsky.feed.defs#replyRef'] = Field(
        default='app.bsky.feed.defs#replyRef', alias='$type', frozen=True
    )


class ReasonRepost(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    by: 'models.AppBskyActorDefs.ProfileViewBasic'  #: By.
    indexed_at: str = Field(alias='indexedAt')  #: Indexed at.

    py_type: te.Literal['app.bsky.feed.defs#reasonRepost'] = Field(
        default='app.bsky.feed.defs#reasonRepost', alias='$type', frozen=True
    )


class ThreadViewPost(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    post: 'models.AppBskyFeedDefs.PostView'  #: Post.
    parent: t.Optional[
        te.Annotated[
            t.Union[
                'models.AppBskyFeedDefs.ThreadViewPost',
                'models.AppBskyFeedDefs.NotFoundPost',
                'models.AppBskyFeedDefs.BlockedPost',
            ],
            Field(default=None, discriminator='py_type'),
        ]
    ] = None  #: Parent.
    replies: t.Optional[
        t.List[
            te.Annotated[
                t.Union[
                    'models.AppBskyFeedDefs.ThreadViewPost',
                    'models.AppBskyFeedDefs.NotFoundPost',
                    'models.AppBskyFeedDefs.BlockedPost',
                ],
                Field(discriminator='py_type'),
            ]
        ]
    ] = None  #: Replies.

    py_type: te.Literal['app.bsky.feed.defs#threadViewPost'] = Field(
        default='app.bsky.feed.defs#threadViewPost', alias='$type', frozen=True
    )


class NotFoundPost(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    not_found: bool = Field(alias='notFound', frozen=True)  #: Not found.
    uri: str  #: Uri.

    py_type: te.Literal['app.bsky.feed.defs#notFoundPost'] = Field(
        default='app.bsky.feed.defs#notFoundPost', alias='$type', frozen=True
    )


class BlockedPost(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    author: 'models.AppBskyFeedDefs.BlockedAuthor'  #: Author.
    blocked: bool = Field(frozen=True)  #: Blocked.
    uri: str  #: Uri.

    py_type: te.Literal['app.bsky.feed.defs#blockedPost'] = Field(
        default='app.bsky.feed.defs#blockedPost', alias='$type', frozen=True
    )


class BlockedAuthor(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    did: str  #: Did.
    viewer: t.Optional['models.AppBskyActorDefs.ViewerState'] = None  #: Viewer.

    py_type: te.Literal['app.bsky.feed.defs#blockedAuthor'] = Field(
        default='app.bsky.feed.defs#blockedAuthor', alias='$type', frozen=True
    )


class GeneratorView(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    cid: str  #: Cid.
    creator: 'models.AppBskyActorDefs.ProfileView'  #: Creator.
    did: str  #: Did.
    display_name: str = Field(alias='displayName')  #: Display name.
    indexed_at: str = Field(alias='indexedAt')  #: Indexed at.
    uri: str  #: Uri.
    avatar: t.Optional[str] = None  #: Avatar.
    description: t.Optional[str] = Field(default=None, max_length=3000)  #: Description.
    description_facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = Field(
        default=None, alias='descriptionFacets'
    )  #: Description facets.
    like_count: t.Optional[int] = Field(default=None, alias='likeCount', ge=0)  #: Like count.
    viewer: t.Optional['models.AppBskyFeedDefs.GeneratorViewerState'] = None  #: Viewer.

    py_type: te.Literal['app.bsky.feed.defs#generatorView'] = Field(
        default='app.bsky.feed.defs#generatorView', alias='$type', frozen=True
    )


class GeneratorViewerState(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    like: t.Optional[str] = None  #: Like.

    py_type: te.Literal['app.bsky.feed.defs#generatorViewerState'] = Field(
        default='app.bsky.feed.defs#generatorViewerState', alias='$type', frozen=True
    )


class SkeletonFeedPost(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    post: str  #: Post.
    reason: t.Optional[
        te.Annotated[
            t.Union['models.AppBskyFeedDefs.SkeletonReasonRepost'], Field(default=None, discriminator='py_type')
        ]
    ] = None  #: Reason.

    py_type: te.Literal['app.bsky.feed.defs#skeletonFeedPost'] = Field(
        default='app.bsky.feed.defs#skeletonFeedPost', alias='$type', frozen=True
    )


class SkeletonReasonRepost(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    repost: str  #: Repost.

    py_type: te.Literal['app.bsky.feed.defs#skeletonReasonRepost'] = Field(
        default='app.bsky.feed.defs#skeletonReasonRepost', alias='$type', frozen=True
    )


class ThreadgateView(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.defs`."""

    cid: t.Optional[str] = None  #: Cid.
    lists: t.Optional[t.List['models.AppBskyGraphDefs.ListViewBasic']] = None  #: Lists.
    record: t.Optional['UnknownType'] = None  #: Record.
    uri: t.Optional[str] = None  #: Uri.

    py_type: te.Literal['app.bsky.feed.defs#threadgateView'] = Field(
        default='app.bsky.feed.defs#threadgateView', alias='$type', frozen=True
    )



import typing as t

from atproto_core.exceptions import AtProtocolError

if t.TYPE_CHECKING:
    from atproto_client.request import Response


class ModelError(AtProtocolError):
    ...


class ModelFieldNotFoundError(ModelError):
    ...


class RequestErrorBase(AtProtocolError):
    def __init__(self, response: t.Optional['Response'] = None) -> None:
        self.response: t.Optional['Response'] = response


class NetworkError(RequestErrorBase):
    ...


class InvokeTimeoutError(NetworkError):
    ...


class UnauthorizedError(RequestErrorBase):
    ...


class RequestException(RequestErrorBase):
    ...


class BadRequestError(RequestErrorBase):
    ...



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.admin.listCommunicationTemplates`."""

    communication_templates: t.List['models.ComAtprotoAdminDefs.CommunicationTemplateView'] = Field(
        alias='communicationTemplates'
    )  #: Communication templates.



P256_DID_PREFIX = b'\x80\x24'
SECP256K1_DID_PREFIX = b'\xe7\x01'

BASE58_MULTIBASE_PREFIX = 'z'
DID_KEY_PREFIX = 'did:key:'

P256_JWT_ALG = 'ES256'
SECP256K1_JWT_ALG = 'ES256K'

P256_CURVE_ORDER = 0xFFFFFFFF_00000000_FFFFFFFF_FFFFFFFF_BCE6FAAD_A7179E84_F3B9CAC2_FC632551
SECP256K1_CURVE_ORDER = 0xFFFFFFFF_FFFFFFFF_FFFFFFFF_FFFFFFFE_BAAEDCE6_AF48A03B_BFD25E8C_D0364141



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.notification.getUnreadCount`."""

    seen_at: t.Optional[str] = Field(default=None, alias='seenAt')  #: Seen at.


class ParamsDict(te.TypedDict):
    seen_at: te.NotRequired[t.Optional[str]]  #: Seen at.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.notification.getUnreadCount`."""

    count: int  #: Count.



import json
import os
import typing as t
from pathlib import Path

from atproto_lexicon import models
from atproto_lexicon.exceptions import LexiconParsingError

_PATH_TO_LEXICONS = Path(__file__).parent.parent.parent.joinpath('lexicons').absolute()
_LEXICON_FILE_EXT = '.json'

L = t.TypeVar('L')


def lexicon_parse(data: dict, model_class: t.Optional[t.Type[L]] = models.LexiconDoc) -> L:
    return model_class(**data)


def lexicon_parse_file(lexicon_path: t.Union[Path, str], *, soft_fail: bool = False) -> t.Optional[models.LexiconDoc]:
    try:
        with open(lexicon_path, encoding='UTF-8') as f:
            plain_lexicon = json.loads(f.read())
            return lexicon_parse(plain_lexicon)
    except Exception as e:  # noqa: BLE001
        if soft_fail:
            return None

        raise LexiconParsingError("Can't parse lexicon") from e


def lexicon_parse_dir(
    lexicon_dir: t.Optional[t.Union[Path, str]] = None, *, soft_fail: bool = False
) -> t.List[models.LexiconDoc]:
    lexicon_dir_path = Path(lexicon_dir) if isinstance(lexicon_dir, str) else lexicon_dir
    if lexicon_dir_path is None:
        lexicon_dir_path = _PATH_TO_LEXICONS

    parsed_lexicons = []

    for _, _, lexicons in sorted(os.walk(lexicon_dir_path)):
        for lexicon in sorted(lexicons):
            if not lexicon.endswith(_LEXICON_FILE_EXT):
                continue

            lexicon_path = lexicon_dir_path.joinpath(lexicon)
            parsed_lexicon = lexicon_parse_file(lexicon_path, soft_fail=soft_fail)
            if parsed_lexicon:
                parsed_lexicons.append(parsed_lexicon)

    return parsed_lexicons



import typing as t

import typing_extensions as te
from pydantic import BaseModel, ConfigDict, Field

Number = t.Union[int, float]


class LexDefinitionType:
    RECORD = 'record'

    QUERY = 'query'
    PROCEDURE = 'procedure'
    SUBSCRIPTION = 'subscription'

    PARAMS = 'params'
    TOKEN = 'token'  # noqa: S105
    OBJECT = 'object'

    BLOB = 'blob'
    ARRAY = 'array'

    STRING = 'string'  # TODO(MarshalX): definitions could be primitives?


class LexPrimitiveType:
    BOOLEAN = 'boolean'
    NUMBER = 'number'
    INTEGER = 'integer'
    STRING = 'string'
    REF = 'ref'
    UNION = 'union'
    UNKNOWN = 'unknown'
    CID_LINK = 'cid-link'
    BYTES = 'bytes'


class LexBase(BaseModel):
    """Base class for all lexicon models."""

    model_config = ConfigDict(extra='forbid', populate_by_name=True, strict=True)


class LexDefinitionBase(LexBase):  # original name LexUserType
    """Base class for all lexicon definitions."""

    description: t.Optional[str] = None


class LexPrimitiveBase(LexBase):
    """Base class for all lexicon primitives."""

    description: t.Optional[str] = None


class LexXrpcError(LexBase):
    name: str
    description: t.Optional[str] = None


class LexUnknown(LexPrimitiveBase):
    type: te.Literal['unknown'] = Field(default=LexPrimitiveType.UNKNOWN, frozen=True)


class LexCidLink(LexPrimitiveBase):
    type: te.Literal['cid-link'] = Field(default=LexPrimitiveType.CID_LINK, frozen=True)


class LexBytes(LexPrimitiveBase):
    type: te.Literal['bytes'] = Field(default=LexPrimitiveType.BYTES, frozen=True)

    max_length: t.Optional[Number] = Field(default=None, alias='maxLength')
    min_length: t.Optional[Number] = Field(default=None, alias='minLength')


class LexBoolean(LexPrimitiveBase):
    type: te.Literal['boolean'] = Field(default=LexPrimitiveType.BOOLEAN, frozen=True)

    default: t.Optional[bool] = None
    const: t.Optional[bool] = None


class LexNumber(LexPrimitiveBase):
    type: te.Literal['number'] = Field(default=LexPrimitiveType.NUMBER, frozen=True)

    default: t.Optional[Number] = None
    minimum: t.Optional[Number] = None
    maximum: t.Optional[Number] = None
    enum: t.Optional[t.List[Number]] = None
    const: t.Optional[Number] = None


class LexInteger(LexPrimitiveBase):
    type: te.Literal['integer'] = Field(default=LexPrimitiveType.INTEGER, frozen=True)

    default: t.Optional[int] = None
    minimum: t.Optional[int] = None
    maximum: t.Optional[int] = None
    enum: t.Optional[t.List[int]] = None
    const: t.Optional[int] = None


class LexString(LexPrimitiveBase):
    type: te.Literal['string'] = Field(default=LexPrimitiveType.STRING, frozen=True)

    format: t.Optional[str] = None
    default: t.Optional[str] = None
    min_length: t.Optional[int] = Field(default=None, alias='minLength')
    max_length: t.Optional[int] = Field(default=None, alias='maxLength')
    min_graphemes: t.Optional[int] = Field(default=None, alias='minGraphemes')
    max_graphemes: t.Optional[int] = Field(default=None, alias='maxGraphemes')
    enum: t.Optional[t.List[str]] = None
    const: t.Optional[str] = None
    known_values: t.Optional[t.List[str]] = Field(default=None, alias='knownValues')


class LexRef(LexPrimitiveBase):
    type: te.Literal['ref'] = Field(default=LexPrimitiveType.REF, frozen=True)

    ref: str


class LexRefUnion(LexPrimitiveBase):
    type: te.Literal['union'] = Field(default=LexPrimitiveType.UNION, frozen=True)

    refs: t.List[str]
    closed: t.Optional[bool] = None


LexRefVariant = te.Annotated[t.Union[LexRef, LexRefUnion], Field(discriminator='type')]
LexPrimitive = te.Annotated[
    t.Union[
        LexUnknown,
        LexCidLink,
        LexBytes,
        LexBoolean,
        LexNumber,
        LexInteger,
        LexString,
        LexRef,
        LexRefVariant,
    ],
    Field(discriminator='type'),
]


class LexBlob(LexDefinitionBase):
    type: te.Literal['blob'] = Field(default=LexDefinitionType.BLOB, frozen=True)

    accept: t.Optional[t.List[str]] = None
    max_size: t.Optional[Number] = Field(default=None, alias='maxSize')


class LexArray(LexDefinitionBase):
    type: te.Literal['array'] = Field(default=LexDefinitionType.ARRAY, frozen=True)

    items: t.Union[LexRefVariant, te.Annotated[t.Union[LexPrimitive, LexBlob], Field(discriminator='type')]]
    min_length: t.Optional[int] = Field(default=None, alias='minLength')
    max_length: t.Optional[int] = Field(default=None, alias='maxLength')


class LexToken(LexDefinitionBase):
    type: te.Literal['token'] = Field(default=LexDefinitionType.TOKEN, frozen=True)


class LexObject(LexDefinitionBase):
    type: te.Literal['object'] = Field(default=LexDefinitionType.OBJECT, frozen=True)

    required: t.Optional[t.List[str]] = None
    nullable: t.Optional[t.List[str]] = None
    properties: t.Dict[str, te.Annotated[t.Union[LexPrimitive, LexArray, LexBlob], Field(discriminator='type')]]


class LexRecord(LexDefinitionBase):
    type: te.Literal['record'] = Field(default=LexDefinitionType.RECORD, frozen=True)

    key: t.Optional[str] = None
    record: LexObject


class LexXrpcParameters(LexDefinitionBase):
    type: te.Literal['params'] = Field(default=LexDefinitionType.PARAMS, frozen=True)

    required: t.Optional[t.List[str]] = None
    properties: t.Dict[str, te.Annotated[t.Union[LexArray, LexPrimitive], Field(discriminator='type')]]


class LexXrpcSubscriptionMessage(LexBase):
    description: t.Optional[str] = None
    schema_: t.Optional[te.Annotated[t.Union[LexObject, LexRefVariant], Field(discriminator='type')]] = Field(
        default=None, alias='schema'
    )


class LexXrpcBody(LexBase):
    description: t.Optional[str] = None
    encoding: t.Union[str, t.List[str]]
    schema_: t.Optional[te.Annotated[t.Union[LexObject, LexRefVariant], Field(discriminator='type')]] = Field(
        default=None, alias='schema'
    )


class LexSubscription(LexDefinitionBase):
    type: te.Literal['subscription'] = Field(default=LexDefinitionType.SUBSCRIPTION, frozen=True)

    parameters: t.Optional[LexXrpcParameters] = None
    message: t.Optional[LexXrpcSubscriptionMessage] = None
    infos: t.Optional[t.List[LexXrpcError]] = None
    errors: t.Optional[t.List[LexXrpcError]] = None


class LexXrpcQuery(LexDefinitionBase):
    type: te.Literal['query'] = Field(default=LexDefinitionType.QUERY, frozen=True)

    parameters: t.Optional[LexXrpcParameters] = None
    output: t.Optional[LexXrpcBody] = None
    errors: t.Optional[t.List[LexXrpcError]] = None


class LexXrpcProcedure(LexDefinitionBase):
    type: te.Literal['procedure'] = Field(default=LexDefinitionType.PROCEDURE, frozen=True)

    parameters: t.Optional[LexXrpcParameters] = None
    input: t.Optional[LexXrpcBody] = None
    output: t.Optional[LexXrpcBody] = None
    errors: t.Optional[t.List[LexXrpcError]] = None


LexDefinition = te.Annotated[
    t.Union[
        LexBlob,
        LexArray,
        LexToken,
        LexObject,
        LexRecord,
        LexXrpcParameters,
        LexSubscription,
        LexXrpcQuery,
        LexXrpcProcedure,
        LexString,  # actually it's primitive
    ],
    Field(discriminator='type'),
]


class LexiconDoc(LexBase):
    """Lexicon document model."""

    lexicon: int
    id: str  # NSID
    defs: t.Dict[str, LexDefinition]
    description: t.Optional[str] = None
    revision: t.Optional[int] = None



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.repo.getRecord`."""

    collection: str  #: The NSID of the record collection.
    repo: str  #: The handle or DID of the repo.
    rkey: str  #: The key of the record.
    cid: t.Optional[
        str
    ] = None  #: The CID of the version of the record. If not specified, then return the most recent version.


class ParamsDict(te.TypedDict):
    collection: str  #: The NSID of the record collection.
    repo: str  #: The handle or DID of the repo.
    rkey: str  #: The key of the record.
    cid: te.NotRequired[
        t.Optional[str]
    ]  #: The CID of the version of the record. If not specified, then return the most recent version.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.repo.getRecord`."""

    uri: str  #: Uri.
    value: 'UnknownType'  #: Value.
    cid: t.Optional[str] = None  #: Cid.



from pathlib import Path

from atproto_codegen.consts import DISCLAIMER
from atproto_codegen.utils import format_code, write_code

_CLIENT_DIR = Path(__file__).parent.parent.parent.joinpath('atproto_client', 'client')


def gen_client(input_filename: str, output_filename: str) -> None:
    with open(_CLIENT_DIR.joinpath(input_filename), 'r', encoding='UTF-8') as f:
        code = f.read()

    # TODO(MarshalX): Get automatically
    methods = [
        'send_post',
        'send_image',
        '_set_session',
        '_get_and_set_session',
        '_refresh_and_set_session',
        '_import_session_string',
        '_call_on_session_change_callbacks',
        '_invoke',
    ]

    code = code.replace('from threading', 'from asyncio')
    code = code.replace('client.raw', 'client.async_raw')
    code = code.replace('class Client', 'class AsyncClient')
    code = code.replace('ClientRaw', 'AsyncClientRaw')
    code = code.replace('SessionDispatchMixin', 'AsyncSessionDispatchMixin')

    code = code.replace('with self', 'async with self')

    code = code.replace('def', 'async def')
    code = code.replace('async def __', 'def __')

    code = code.replace('self.com', 'await self.com')
    code = code.replace('self.app', 'await self.app')

    for method in methods:
        code = code.replace(f'self.{method}(', f'await self.{method}(')
        code = code.replace(f'super().{method}(', f'await super().{method}(')

    code = DISCLAIMER + code

    write_code(_CLIENT_DIR.joinpath(output_filename), code)
    format_code(_CLIENT_DIR.joinpath(output_filename))



import json
import typing as t
from dataclasses import dataclass

import httpx

from atproto_client import exceptions
from atproto_client.models.common import XrpcError
from atproto_client.models.utils import get_or_create, is_json


@dataclass
class Response:
    success: bool
    status_code: int
    content: t.Optional[t.Union[dict, bytes, 'XrpcError']]
    headers: t.Dict[str, t.Any]


def _parse_response(response: httpx.Response) -> Response:
    content = response.content
    if response.headers.get('content-type') == 'application/json; charset=utf-8':
        content = response.json()

    return Response(
        success=True,
        status_code=response.status_code,
        content=content,
        headers=response.headers,
    )


def _handle_request_errors(exception: Exception) -> None:
    try:
        raise exception
    except httpx.TimeoutException as e:
        raise exceptions.InvokeTimeoutError from e
    except httpx.NetworkError as e:
        raise exceptions.NetworkError from e
    # TODO(MarshalX): add more exceptions


def _handle_response(response: httpx.Response) -> httpx.Response:
    if 200 <= response.status_code <= 299:
        return response

    error_response = Response(
        success=False,
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
    )
    if response.content and is_json(response.content):
        data = json.loads(response.content)
        error_response.content = get_or_create(data, XrpcError)

    if response.status_code in {401, 403}:
        raise exceptions.UnauthorizedError(error_response)
    if response.status_code == 400:
        raise exceptions.BadRequestError(error_response)
    if response.status_code in {409, 413, 502}:
        raise exceptions.NetworkError(error_response)

    raise exceptions.RequestException(error_response)


class RequestBase:
    _MANDATORY_HEADERS: t.ClassVar[t.Dict[str, str]] = {'User-Agent': 'atproto/alpha (Python SDK)'}

    def __init__(self) -> None:
        self._additional_headers: dict = {}

    def get_headers(self, additional_headers: t.Optional[dict] = None) -> dict:
        headers = {**RequestBase._MANDATORY_HEADERS, **self._additional_headers}

        if additional_headers:
            headers.update(additional_headers)

        return headers

    def set_additional_headers(self, headers: dict) -> None:
        if isinstance(headers, dict):
            self._additional_headers = headers.copy()
        else:
            raise ValueError('Headers must be dict')


class Request(RequestBase):
    """Class for handling requests errors and working with httpx."""

    def __init__(self) -> None:
        super().__init__()
        self._client = httpx.Client()

    def _send_request(self, method: str, url: str, **kwargs: t.Any) -> httpx.Response:
        headers = self.get_headers(kwargs.pop('headers', None))

        try:
            response = self._client.request(method=method, url=url, headers=headers, **kwargs)
            return _handle_response(response)
        except Exception as e:
            _handle_request_errors(e)
            raise e

    def close(self) -> None:
        self._client.close()

    def get(self, *args, **kwargs: t.Any) -> Response:
        return _parse_response(self._send_request('GET', *args, **kwargs))

    def post(self, *args, **kwargs: t.Any) -> Response:
        return _parse_response(self._send_request('POST', *args, **kwargs))


class AsyncRequest(RequestBase):
    """Class for handling requests errors and working with httpx."""

    def __init__(self) -> None:
        super().__init__()
        self._client = httpx.AsyncClient()

    async def _send_request(self, method: str, url: str, **kwargs: t.Any) -> httpx.Response:
        headers = self.get_headers(kwargs.pop('headers', None))

        try:
            response = await self._client.request(method=method, url=url, headers=headers, **kwargs)
            return _handle_response(response)
        except Exception as e:
            _handle_request_errors(e)
            raise e

    async def close(self) -> None:
        await self._client.aclose()

    async def get(self, *args, **kwargs: t.Any) -> Response:
        return _parse_response(await self._send_request('GET', *args, **kwargs))

    async def post(self, *args, **kwargs: t.Any) -> Response:
        return _parse_response(await self._send_request('POST', *args, **kwargs))



import typing as t

import typing_extensions as te
from atproto_core.cid import CID
from pydantic import BaseModel, ConfigDict, Field


class IpldLink(BaseModel):
    """CID representation in JSON."""

    link: str = Field(alias='$link')  #: CID.


class BlobRef(BaseModel):
    """Blob reference."""

    model_config = ConfigDict(extra='allow', populate_by_name=True, strict=True)

    mime_type: str = Field(alias='mimeType')  #: Mime type.
    size: int  #: Size in bytes.
    ref: t.Union[str, IpldLink]  #: CID.

    py_type: te.Literal['blob'] = Field(default='blob', alias='$type')

    @property
    def cid(self) -> 'CID':
        """Get CID."""
        if isinstance(self.ref, str):
            return CID.decode(self.ref)

        return CID.decode(self.ref.link)



from atproto_client import models
from atproto_client.client.async_client import AsyncClient
from atproto_client.client.client import Client
from atproto_client.client.session import Session, SessionEvent

__all__ = [
    'AsyncClient',
    'Client',
    'models',
    'SessionEvent',
    'Session',
]
